@@ -36,17 +36,13 @@ use tor_rtcompat::{Runtime, SleepProviderExt};
 #[cfg(feature = "onion-service-client")]
 use {
     tor_config::BoolOrAuto,
-    tor_hsclient::{
-        HsClientConnector, HsClientDescEncKeypairSpecifier, HsClientNickname,
-        HsClientSecretKeysBuilder,
-    },
+    tor_hsclient::{HsClientConnector, HsClientDescEncKeypairSpecifier, HsClientSecretKeysBuilder},
     tor_hscrypto::pk::HsClientDescEncKeypair,
     tor_netdir::DirEvent,
 };
 
 use tor_keymgr::{ArtiNativeKeystore, KeyMgr, KeyMgrBuilder};
 
-use educe::Educe;
 use futures::lock::Mutex as AsyncMutex;
 use futures::task::SpawnExt;
 use futures::StreamExt as _;
@@ -182,13 +178,12 @@ pub struct TorClient<R: Runtime> {
 }
 
 /// Preferences for whether a [`TorClient`] should bootstrap on its own or not.
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Educe)]
-#[educe(Default)]
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq)]
 #[non_exhaustive]
 pub enum BootstrapBehavior {
     /// Bootstrap the client automatically when requests are made that require the client to be
     /// bootstrapped.
-    #[educe(Default)]
+    #[default]
     OnDemand,
     /// Make no attempts to automatically bootstrap. [`TorClient::bootstrap`] must be manually
     /// invoked in order for the [`TorClient`] to become useful.
@@ -200,12 +195,11 @@ pub enum BootstrapBehavior {
 }
 
 /// What level of sleep to put a Tor client into.
-#[derive(Debug, Copy, Clone, Educe, PartialEq, Eq)]
-#[educe(Default)]
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq)]
 #[non_exhaustive]
 pub enum DormantMode {
     /// The client functions as normal, and background tasks run periodically.
-    #[educe(Default)]
+    #[default]
     Normal,
     /// Background tasks are suspended, conserving CPU usage. Attempts to use the client will
     /// wake it back up again.
@@ -213,8 +207,7 @@ pub enum DormantMode {
 }
 
 /// Preferences for how to route a stream over the Tor network.
-#[derive(Debug, Clone, Educe)]
-#[educe(Default)]
+#[derive(Debug, Default, Clone)]
 pub struct StreamPrefs {
     /// What kind of IPv6/IPv4 we'd prefer, and how strongly.
     ip_ver_pref: IpVersionPreference,
@@ -238,11 +231,10 @@ pub struct StreamPrefs {
 }
 
 /// Record of how we are isolating connections
-#[derive(Debug, Clone, Educe)]
-#[educe(Default)]
+#[derive(Debug, Default, Clone)]
 enum StreamIsolationPreference {
     /// No additional isolation
-    #[educe(Default)]
+    #[default]
     None,
     /// Isolation parameter to use for connections
     Explicit(Box<dyn Isolation>),
@@ -1060,11 +1052,7 @@ impl<R: Runtime> TorClient<R> {
                 let mut hs_client_secret_keys_builder = HsClientSecretKeysBuilder::default();
 
                 if let Some(keymgr) = &self.keymgr {
-                    // TODO hs: use a real client id (loaded from the config)
-                    let client_id = HsClientNickname::new("default".into())
-                        .map_err(ErrorDetail::BadClientSpecifier)?;
-                    let desc_enc_key_spec =
-                        HsClientDescEncKeypairSpecifier::new(client_id.clone(), hsid);
+                    let desc_enc_key_spec = HsClientDescEncKeypairSpecifier::new(hsid);
 
                     // TODO hs: refactor to reduce code duplication.
                     //
@@ -1436,11 +1424,12 @@ impl<R: Runtime> TorClient<R> {
     fn create_keymgr(config: &TorClientConfig) -> StdResult<Option<Arc<KeyMgr>>, ErrorDetail> {
         let keystore = config.storage.keystore();
         if keystore.is_enabled() {
-            let key_store_dir = keystore.path();
+            let (state_dir, _mistrust) = Self::state_dir(config)?;
+            let key_store_dir = state_dir.join("keystore");
             let permissions = config.storage.permissions();
 
             let arti_store =
-                ArtiNativeKeystore::from_path_and_mistrust(key_store_dir, permissions)?;
+                ArtiNativeKeystore::from_path_and_mistrust(&key_store_dir, permissions)?;
             info!("Using keystore from {key_store_dir:?}");
 
             // TODO #1106: make the default store configurable
