@@ -211,6 +211,7 @@ impl<R: Runtime> HsCircPool<R> {
             .map_err(|error| Error::Protocol {
                 action: "extending to chosen HS hop",
                 peer: None, // Either party could be to blame.
+                unique_id: Some(circ.unique_id()),
                 error,
             });
 
@@ -220,7 +221,7 @@ impl<R: Runtime> HsCircPool<R> {
             .peek_runtime()
             .timeout(extend_timeout, extend_future)
             .await
-            .map_err(|_| Error::CircTimeout)??;
+            .map_err(|_| Error::CircTimeout(Some(circ.unique_id())))??;
 
         // With any luck, return the circuit.
         Ok(circ)
@@ -334,6 +335,8 @@ struct TargetInfo<'a> {
 impl<'a> TargetInfo<'a> {
     /// Return true if, according to the rules of `subnet_config`, this target can share a circuit with `r`.
     fn may_share_circuit_with(&self, r: &Relay<'_>, subnet_config: SubnetConfig) -> bool {
+        // TODO #504
+        // TODO 768
         if let Some(this_r) = &self.relay {
             if this_r.in_same_family(r) {
                 return false;
