@@ -3,6 +3,7 @@
 mod backoff;
 mod descriptor;
 mod reactor;
+mod reupload_timer;
 
 use futures::task::SpawnExt;
 use postage::watch;
@@ -247,7 +248,7 @@ mod test {
         /// The responses that will be returned by each test HSDir (identified by its RsaIdentity).
         ///
         /// Used for testing whether the reactor correctly retries on failure.
-        responses_for_hsdir: Arc<Mutex<HashMap<rsa::RsaIdentity, Arc<Mutex<I>>>>>,
+        responses_for_hsdir: Arc<Mutex<HashMap<rsa::RsaIdentity, I>>>,
     }
 
     #[async_trait]
@@ -275,11 +276,11 @@ mod test {
             let mut map = self.responses_for_hsdir.lock().unwrap();
             let poll_read_responses = map
                 .entry(*id)
-                .or_insert_with(|| Arc::new(Mutex::new(self.poll_read_responses.clone())));
+                .or_insert_with(|| self.poll_read_responses.clone());
 
             Ok(MockClientCirc {
                 publish_count: Arc::clone(&self.publish_count),
-                poll_read_responses: Arc::clone(poll_read_responses),
+                poll_read_responses: poll_read_responses.clone(),
             }
             .into())
         }
@@ -297,7 +298,7 @@ mod test {
         /// The values to return from `poll_read`.
         ///
         /// Used for testing whether the reactor correctly retries on failure.
-        poll_read_responses: Arc<Mutex<I>>,
+        poll_read_responses: I,
     }
 
     #[async_trait]
@@ -309,7 +310,7 @@ mod test {
                 publish_count: Arc::clone(&self.publish_count),
                 // TODO: this will need to change when we start reusing circuits (currently,
                 // we only ever create one data stream per circuit).
-                poll_read_responses: Arc::clone(&self.poll_read_responses),
+                poll_read_responses: self.poll_read_responses.clone(),
             })
         }
     }
@@ -321,16 +322,16 @@ mod test {
         /// The values to return from `poll_read`.
         ///
         /// Used for testing whether the reactor correctly retries on failure.
-        poll_read_responses: Arc<Mutex<I>>,
+        poll_read_responses: I,
     }
 
     impl<I: PollReadIter> AsyncRead for MockDataStream<I> {
         fn poll_read(
-            self: Pin<&mut Self>,
+            mut self: Pin<&mut Self>,
             cx: &mut Context<'_>,
             buf: &mut [u8],
         ) -> Poll<io::Result<usize>> {
-            match self.poll_read_responses.lock().unwrap().next() {
+            match self.as_mut().poll_read_responses.next() {
                 Some(res) => {
                     match res {
                         Ok(res) => {
@@ -467,6 +468,7 @@ mod test {
         reactor_event: impl FnOnce(),
         poll_read_responses: I,
         expected_upload_count: usize,
+        republish_count: usize,
     ) {
         runtime.clone().block_on(async move {
             let netdir_provider: Arc<dyn NetDirProvider> =
@@ -490,16 +492,46 @@ mod test {
             );
 
             publisher.launch().unwrap();
-            runtime.advance_until_stalled().await;
+            runtime.progress_until_stalled().await;
 
             // Check that we haven't published anything yet
             assert_eq!(publish_count.load(Ordering::SeqCst), 0);
 
             reactor_event();
 
-            runtime.advance_until_stalled().await;
+            runtime.progress_until_stalled().await;
+            // We need to manually advance the time, because some of our tests check that the
+            // failed uploads are retried, and there's a sleep() between the retries
+            // (see BackoffSchedule::next_delay).
+            runtime.advance_by(Duration::from_secs(1)).await;
+            runtime.progress_until_stalled().await;
 
-            assert_eq!(publish_count.load(Ordering::SeqCst), expected_upload_count);
+            let initial_publish_count = publish_count.load(Ordering::SeqCst);
+            assert_eq!(initial_publish_count, expected_upload_count);
+
+            if republish_count > 0 {
+                /// The earliest time the descriptor can be republished.
+                const MIN_TIMEOUT: Duration = Duration::from_secs(60 * 60);
+                /// The latest time the descriptor can be republished.
+                const MAX_TIMEOUT: Duration = Duration::from_secs(60 * 120);
+
+                // Wait until the reactor triggers the necessary number of reuploads.
+                runtime
+                    .advance_by(MAX_TIMEOUT * (republish_count as u32))
+                    .await;
+                runtime.progress_until_stalled().await;
+
+                let min_upload_count = expected_upload_count * republish_count;
+                // There will be twice as many reuploads if the publisher happens
+                // to reupload every hour (as opposed to every 2h).
+                let max_upload_count = 2 * min_upload_count;
+                let publish_count_now = publish_count.load(Ordering::SeqCst);
+                // This is the total number of reuploads (i.e. the number of times
+                // we published the descriptor to an HsDir).
+                let actual_reupload_count = publish_count_now - initial_publish_count;
+
+                assert!((min_upload_count..=max_upload_count).contains(&actual_reupload_count));
+            }
         });
     }
 
@@ -516,6 +548,7 @@ mod test {
         temp_dir: &Path,
         poll_read_responses: I,
         multiplier: usize,
+        republish_count: usize,
     ) {
         let runtime = MockRuntime::new();
         let nickname = HsNickname::try_from(TEST_SVC_NICKNAME.to_string()).unwrap();
@@ -571,6 +604,7 @@ mod test {
             update_ipts,
             poll_read_responses,
             expected_upload_count,
+            republish_count,
         );
     }
 
@@ -579,7 +613,7 @@ mod test {
         // The HSDirs always respond with 200 OK, so we expect to publish hsdir_count times.
         let poll_reads = [Ok(OK_RESPONSE.into())].into_iter();
 
-        test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1));
+        test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, 0));
     }
 
     #[test]
@@ -603,10 +637,20 @@ mod test {
             ]
             .into_iter();
 
-            test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 2));
+            test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 2, 0));
         }
     }
 
+    #[test]
+    fn reupload_after_publishing() {
+        let poll_reads = [Ok(OK_RESPONSE.into())].into_iter();
+        // Test that 4 reuploads happen after the initial upload
+        const REUPLOAD_COUNT: usize = 4;
+
+        test_temp_dir!()
+            .used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, REUPLOAD_COUNT));
+    }
+
     // TODO (#1120): test that the descriptor is republished when the config changes
 
     // TODO (#1120): test that the descriptor is reuploaded only to the HSDirs that need it (i.e. the
