@@ -40,7 +40,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
-// TODO (#1115): write more comprehensive documentation when the API is a bit more
+// TODO: write more comprehensive documentation when the API is a bit more
 // stable
 
 mod arti_path;
@@ -63,9 +63,10 @@ mod dummy;
 pub use arti_path::{ArtiPath, DENOTATOR_SEP};
 pub use err::{ArtiPathSyntaxError, Error, KeystoreCorruptionError, KeystoreError};
 pub use key_specifier::{
-    ArtiPathUnavailableError, CTorPath, KeyInfoExtractor, KeyPath, KeyPathError, KeyPathInfo,
-    KeyPathInfoBuilder, KeyPathPattern, KeyPathRange, KeySpecifier, KeySpecifierComponent,
-    KeySpecifierComponentViaDisplayFromStr, KeySpecifierPattern,
+    ArtiPathUnavailableError, CTorPath, InvalidKeyPathComponentValue, KeyPath, KeyPathError,
+    KeyPathInfo, KeyPathInfoBuilder, KeyPathInfoExtractor, KeyPathPattern, KeyPathRange,
+    KeySpecifier, KeySpecifierComponent, KeySpecifierComponentViaDisplayFromStr,
+    KeySpecifierPattern,
 };
 
 #[cfg(feature = "keymgr")]
@@ -74,7 +75,7 @@ pub use {
     key_type::{KeyType, UnknownKeyTypeError},
     keystore::arti::ArtiNativeKeystore,
     keystore::{EncodableKey, ErasedKey, Keygen, KeygenRng, Keystore, SshKeyData, ToEncodableKey},
-    mgr::{KeyMgr, KeyMgrBuilder},
+    mgr::{KeyMgr, KeyMgrBuilder, KeystoreEntry},
     ssh_key,
 };
 
@@ -91,7 +92,7 @@ pub(crate) type BoxedKeystore = Box<dyn Keystore>;
 #[doc(hidden)]
 pub use {derive_adhoc, inventory};
 
-use derive_more::{AsRef, Display};
+use derive_more::{AsRef, Display, From};
 use serde::{Deserialize, Serialize};
 use std::str::FromStr;
 
@@ -117,7 +118,7 @@ impl FromStr for KeystoreId {
 }
 
 /// Specifies which keystores a [`KeyMgr`] operation should apply to.
-#[derive(Copy, Clone, Default, Debug, PartialEq, Eq, Hash)]
+#[derive(Copy, Clone, Default, Debug, PartialEq, Eq, Hash, From)]
 #[non_exhaustive]
 pub enum KeystoreSelector<'a> {
     /// Try to use the keystore with the specified ID.
